/********************************************************************************************
 *                                                                                          *
 * Copyright (C) 2017 Armin Felder, Dennis Beier                                            *
 * This file is part of RocketChatMobileEngine <https://git.fairkom.net/chat/fairchat>.     *
 *                                                                                          *
 * RocketChatMobileEngine is free software: you can redistribute it and/or modify           *
 * it under the terms of the GNU General Public License as published by                     *
 * the Free Software Foundation, either version 3 of the License, or                        *
 * (at your option) any later version.                                                      *
 *                                                                                          *
 * RocketChatMobileEngine is distributed in the hope that it will be useful,                *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                            *
 * GNU General Public License for more details.                                             *
 *                                                                                          *
 * You should have received a copy of the GNU General Public License                        *
 * along with RocketChatMobileEngine. If not, see <http://www.gnu.org/licenses/>.           *
 *                                                                                          *
 ********************************************************************************************/



#ifndef CHANNELREPOSITORY_H
#define CHANNELREPOSITORY_H

#include <QObject>
#include "utils.h"
#include "abstractbaserepository.h"
#include "repos/entities/rocketchatchannel.h"
#include "CustomModels/channelmodel.h"
#include "CustomModels/messagemodel.h"
class MessagesModel;
class RocketChatChannel;
class ChannelModel;

class ChannelRepository : public QObject, public AbstractBaseRepository<RocketChatChannel>
{
        Q_OBJECT
    public:

        explicit ChannelRepository( ChannelModel &channelsModel, ChannelModel &groupsModel, ChannelModel &directModel,
                                    MessagesModel &messagesModel );
        unsigned long  getYoungestMessageDate();
        unsigned long  getOldestMessageDate();
        bool add( QString id, QSharedPointer<RocketChatChannel> );
        bool add( QSharedPointer<RocketChatChannel> pChannel );
        void receiveTimestamp( QString pId, qint64 pTimestamp );
        QSharedPointer<RocketChatChannel> getChannelByName( QString pName );
        ~ChannelRepository();
    private:
        QHash<QSharedPointer<RocketChatChannel>, qint64 >  mChannelTimeIndex;
        QMap<qint64, QSharedPointer<RocketChatChannel> > mTimestampIndex;
        ChannelModel &mChannelsModel, &mGroupsModel, &mDirectModel;
        QHash<QString, QSharedPointer<RocketChatChannel> > mNameIndex;
        MessagesModel &mMessagesModel;

    signals:
        void orderChanged( QString channelId );
};

#endif // CHANNELREPOSITORY_H
